use super::{error::PEParseError, header::*};
use std::ops::{Deref, DerefMut};

pub trait PE: Deref<Target = [u8]> + DerefMut<Target = [u8]> + Sized {
    /// Get a reference to a type at a given offset.
    fn get_ref<T>(&self, offset: usize) -> Result<&T, PEParseError> {
        let len = std::mem::size_of::<T>();
        if offset + len > self.len() {
            return Err(PEParseError::OutOfBounds);
        }
        let ptr = self.as_ptr().wrapping_offset(offset as isize) as *const T;
        Ok(unsafe { &*ptr })
    }

    /// Get a mutable reference to a type at a given offset.
    fn get_mut<T>(&mut self, offset: usize) -> Result<&mut T, PEParseError> {
        let len = std::mem::size_of::<T>();
        if offset + len > self.len() {
            return Err(PEParseError::OutOfBounds);
        }
        let ptr = self.as_mut_ptr().wrapping_offset(offset as isize) as *mut T;
        Ok(unsafe { &mut *ptr })
    }

    /// Get the DOS header without verifying its contents.
    fn get_dos_header(&self) -> Result<&ImageDosHeader, PEParseError> {
        let result = self.get_ref::<ImageDosHeader>(0)?;
        Ok(result)
    }
    /// Get the DOS header without mutating its contents.
    fn get_dos_header_mut(&mut self) -> Result<&mut ImageDosHeader, PEParseError> {
        let result = self.get_mut::<ImageDosHeader>(0)?;
        Ok(result)
    }

    /// Get the NT headers without verifying its contents.
    fn get_nt_headers(&self) -> Result<&ImageNTHeaders32, PEParseError> {
        let nt_offset = self.get_nt_headers_offset()?;
        let result = self.get_ref::<ImageNTHeaders32>(nt_offset)?;
        Ok(result)
    }

    // 获取nt头的偏移
    fn get_nt_headers_offset(&self) -> Result<usize, PEParseError> {
        let dos_header = self.get_dos_header()?;
        let nt_offset = dos_header.e_lfanew.0 as usize;
        Ok(nt_offset)
    }

    /// Get the NT headers without mutating its contents.
    fn get_nt_headers_mut(&mut self) -> Result<&mut ImageNTHeaders32, PEParseError> {
        let nt_offset = self.get_nt_headers_offset()?;
        let result = self.get_mut::<ImageNTHeaders32>(nt_offset)?;
        Ok(result)
    }

    // 获取文件头的偏移
    fn get_file_header_offset(&self) -> Result<usize, PEParseError> {
        // 1. 获取nt头偏移
        let nt_offset = self.get_nt_headers_offset()?;
        // 2. 计算文件头偏移
        let file_header_offset = nt_offset + std::mem::size_of::<u32>();
        Ok(file_header_offset)
    }

    // 获取文件头数据
    fn get_file_header(&self) -> Result<&ImageFileHeader, PEParseError> {
        let file_header_offset = self.get_file_header_offset()?;
        let result = self.get_ref::<ImageFileHeader>(file_header_offset)?;
        Ok(result)
    }
    // 获取文件头数据
    fn get_file_header_mut(&mut self) -> Result<&mut ImageFileHeader, PEParseError> {
        let file_header_offset = self.get_file_header_offset()?;
        let result = self.get_mut::<ImageFileHeader>(file_header_offset)?;
        Ok(result)
    }


}
